/* tosserial.c: tty line interface code for Pilot comms under TOS
 *
 * Copyright (c) 1999 Bodo Wenzel
 * This is free software, licensed under the GNU Library Public License V2.
 * See the file COPYING.LIB for details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <tos.h>
#include "pi-source.h"
#include "pi-socket.h"
#include "pi-slp.h"
#include "pi-serial.h"

#define MODEM1 (1) /* device number for BIOS calls */

static int calcrate(long baudrate) {
  if(baudrate == 300) return 9;
  if(baudrate == 600) return 8;
  if(baudrate == 1200) return 7;
  if(baudrate == 2400) return 4;
  if(baudrate == 4800) return 2;
  if(baudrate == 9600) return 1;
  if(baudrate == 19200) return 0;
  printf("Unable to set baud rate %d\n", baudrate);
  abort(); /* invalid baud rate */
  return -1;
}

static int s_changebaud(struct pi_socket *ps);
static int s_close(struct pi_socket *ps);
static int s_write(struct pi_socket *ps);
static int s_read(struct pi_socket *ps, int timeout);

int pi_serial_open(struct pi_socket *ps, struct pi_sockaddr * addr, socketlen_t addrlen)
{
  char * tty = addr->pi_device;

  (void)addrlen;

  /* open the device */
  if ((!tty) || !strlen(tty))
    tty = getenv("PILOTPORT");
  if (!tty)
    tty = "<Null>";

  if (strcmp(tty,"Modem1")!=0) {
    errno = EINVAL;
    return -1;
  }

  /* Set the tty to 8N1 and to the correct speed */
  Rsconf(calcrate(ps->rate),0,0x88,0x01,0x05,0);

  /* flush input */
  while (Bconstat(MODEM1))
    (void)Bconin(MODEM1);

  ps->mac->fd=OPEN_MAX+1; /* just as place holder */

  ps->serial_close = s_close;
  ps->serial_read = s_read;
  ps->serial_write = s_write;
  ps->serial_changebaud = s_changebaud;
  
#ifndef NO_SERIAL_TRACE
  if (ps->debuglog) {
    ps->debugfd = open(ps->debuglog,O_WRONLY|O_CREAT,0666);
    /* This sequence is magic used by my trace analyzer - kja */
    write(ps->debugfd, "\0\1\0\0\0\0\0\0\0\0", 10);
  }
#endif

  return 0;
}


static int s_changebaud(struct pi_socket *ps)
{
  Rsconf(calcrate(ps->rate),0,-1,-1,-1,-1);
  return 0;
}


static int s_close(struct pi_socket *ps)
{
  /* flush left input */
  while (Bconstat(MODEM1))
    (void)Bconin(MODEM1);

#ifndef NO_SERIAL_TRACE
  if (ps->debugfd)
    close(ps->debugfd);  
#endif

  ps->mac->fd = 0;
  return 0;
}


static int s_write(struct pi_socket *ps)
{
  struct pi_skb *skb;
  int i;

  if (ps->txq) {
    ps->busy++;
    skb = ps->txq;
    ps->txq = skb->next;
    for (i=0;i<skb->len;i++) {
      Bconout(MODEM1,skb->data[i]);
    }
#ifndef NO_SERIAL_TRACE
    if (ps->debuglog)
      for (i=0;i<skb->len;i++) {
        write(ps->debugfd, "2", 1);
        write(ps->debugfd, skb->data+i, 1);
      }
#endif
    ps->tx_bytes += skb->len;
    free(skb);

    ps->busy--;
    
    return 1;
  }
  return 0;
}


static int s_read(struct pi_socket *ps, int timeout)
{
  unsigned char *buf;
  int i;
  clock_t t;

  /* If timeout == 0, wait forever for packet, otherwise wait till
     timeout milliseconds */
  /* So it's intended, but because of TOS' inability we set some time */
#define NONULLTIMEOUT (30*1000)
#ifdef NONULLTIMEOUT
  if (timeout == 0)
    timeout = NONULLTIMEOUT;
#endif

  pi_serial_flush(ps);              /* We likely want to be in sync with tx */
  if (!ps->mac->expect) slp_rx(ps);  /* let SLP know we want a packet */

  while (ps->mac->expect) {
    buf = ps->mac->buf;

    for (i=0;i<ps->mac->expect;i++) {
#ifdef NONULLTIMEOUT
      if (timeout != 0)
#endif
      {
        /* If data is available in time, read it */
        t = clock() + CLK_TCK*(clock_t)timeout/1000;
        while (!Bconstat(MODEM1)) {
          if (t<clock()) {
            /* otherwise throw out any current packet and return */
            ps->mac->state = ps->mac->expect = 1;
            ps->mac->buf = ps->mac->rxb->data;
            ps->rx_errors++;
            return 0;
          }
        }
      }
      *buf++ = (unsigned char)Bconin(MODEM1);
    }
#ifndef NO_SERIAL_TRACE
    if (ps->debuglog)
      for (i=0;i<ps->mac->expect;i++) {
        write(ps->debugfd, "1", 1);
        write(ps->debugfd, buf+i, 1);
      }
#endif
    ps->rx_bytes += ps->mac->expect;
    ps->mac->expect = 0;

    slp_rx(ps);
  }
  return 0;
}
